from bitcoding import B_dict
from basesplit import bsplit as split
from inverse import inv
from glob import globs
from vbweights import w
from collections import deque
import random

random.seed(21)

if globs['pcheuristic'] == 0: # path consistency
   from pc import PC as pc 
elif globs['pcheuristic'] == 1: # path consistency with weighted queue
   from pcw import PCw as pc

def extractConScen(solution, neighbors, frozen):

   pq = deque([])                         # list of entries arranged in a heap
   entry_finder = set([])               # mapping of tasks to entries
   Vars = len(solution)

   def add_task(task):
       'Add a new task or update the priority of an existing task'
       if task not in entry_finder:
          entry_finder.add(task)
          pq.append(task)
          
   def pop_task():
       'Remove and return the lowest priority task. Raise KeyError if empty.'
       task = pq.popleft()
       entry_finder.discard(task)
       return task

   # initialize queue 
   for i in range(Vars):
      for j in range(i+1,Vars):
         if split[solution[i][j]-1][0] > 1 and (i,j) not in frozen:
            add_task((i,j))  

   # as long as the queue is not empty, process it
   while pq:

      (i,j) = pop_task() # grab the appropriate relation
      
      # splitting
      values = split[solution[i][j]-1][1][:]

      #least constraining value heuristic
      #valuesw = [(-w[a-1],a) for a in values]
      #valuesw.sort(reverse=True)
      #valuesw = [a[1] for a in valuesw]
      
      valuesw = values[:]
      random.shuffle(valuesw)
      
      while valuesw:
      
         value = valuesw.pop() # grab first value from variable
      
         repairs = {}
         repairs[(i,j)] = solution[i][j]
         solution[i][j] = value
         solution[j][i] = inv[value-1]                  
         result, _ = pc(solution, neighbors, frozen, i, j, repairs, [], False)
         if not result:
            for g, h in repairs.keys():
               solution[g][h] = repairs[(g, h)] 
               solution[h][g] = inv[repairs[(g, h)]-1]
         else:
            break
      else:
         raise RuntimeError("Can't happen")
            
   return solution


