#!/usr/bin/env python3

import time
import sys

def fileLen(fname):
    with open(fname) as f:
        for i, l in enumerate(f):
            pass
    return i + 1

def init(buffer):
   from glob import globs, setGlobals
   from array import array

   # create spatial variables of specified number
   temp = buffer.pop(0)
   Vars = int(temp.split()[0].strip())+1
   TypeId = temp.split()[1].strip()
   
   # initialize constraints
   N = fileLen(globs['calc']+"/"+globs['calc']+".relations")
   setGlobals("size", N)
   Id = 0
   from bitcoding import B_dict
   with open(globs['calc']+"/"+globs['calc']+".identity") as f:
      Id = B_dict[f.readline().strip()]
   if N <= 8:
      ConMatrix = tuple([array('B',[B_dict['DALL'] if i != j else Id for i in range(Vars)]) for j in range(Vars)])
   elif N <= 16:
      ConMatrix = tuple([array('H',[B_dict['DALL'] if i != j else Id for i in range(Vars)]) for j in range(Vars)])
   else:
      ConMatrix = tuple([array('I',[B_dict['DALL'] if i != j else Id for i in range(Vars)]) for j in range(Vars)])
  
   # parse spatial CSP
   from parsecsp import parsecsp
   parsecsp(ConMatrix, buffer)
   from comptab import fcomp

   return TypeId, ConMatrix

class Usage(Exception):
    def __init__(self, msg):
        self.msg = msg

# main function
def main(argv=None):

   if argv is None:
        argv = sys.argv

   # set some sys useful stuff
   sys.setrecursionlimit(1<<30)
   #sys.setcheckinterval(10000)

   from argparse import ArgumentParser

   # initialize parser for command line arguments
   parser = ArgumentParser()
   
   # set parsing options
   parser.add_argument("-c", "--calculus", type=str, dest="calc", default="rcc8", help="interaction mode: rcc8, or allen [default: %(default)s]")
   parser.add_argument("-m", "--method", type=str, dest="method", default="iterative", help="interaction mode: recursive, or iterative [default: %(default)s]")
   parser.add_argument("-p", "--pcheuristic", type=str, dest="pcheuristic", default="none", help="interaction mode: none, or weighted [default: %(default)s]")
   parser.add_argument("-s", "--split", type=str, dest="split", default="horn", help="interaction mode: base, or horn [default: %(default)s]")
   parser.add_argument("-d", "--print", action="store_true", dest="printsol", help="print solution to output")
   parser.add_argument("-z", "--pc", action="store_true", dest="pc", help="path consistency only")
   parser.add_argument("-e", "--heuristic", type=str, dest="heuristic", default="global", help="interaction mode: global, or none [default: %(default)s]")
   parser.add_argument("-f", "--file", action="store", type=str, dest="filename")

   # parse command line arguments
   args = parser.parse_args()

   from glob import setGlobals

   try:   
      # check and set globals for spliting information
      if args.split == "base":
         setGlobals('split',0)
      elif args.split == "horn":
         setGlobals('split',1)
      else:
         raise Usage("Invalid option: " + args.split)

      # check and set globals for calculus information
      if not (args.calc == "rcc8" or args.calc == "allen"):
         raise Usage("Invalid option: " + args.calc)
      else:
         setGlobals('calc',args.calc)

      # check and set globals for patch consistency heuristic information
      if args.pcheuristic == "none":
         setGlobals('pcheuristic',0)
      elif args.pcheuristic == "weighted":
         setGlobals('pcheuristic',1)
      else:
         raise Usage("Invalid option: " + args.pcheuristic)

      # check and set globals for consistency heuristic information
      if args.heuristic == "none":
         setGlobals('heuristic',0)
      elif args.heuristic == "global":
         setGlobals('heuristic',2)
      else:
         raise Usage("Invalid option: " + args.heuristic)

      # check and set globals for method decision information
      if args.method == "recursive":
         setGlobals('method','rec')
      elif args.method == "iterative": 
         setGlobals('method','iter')
      else:
         raise Usage("Invalid option: " + args.method)

   except Usage as err:
        print(err.msg)
        parser.print_help()
        return 1

   with open(args.filename) as f:
      total = 0
      meter = 0
      buffer = []
      plevels = []
      decs = []
      nods = []
      times = []
      satcalls = []

      for i in f:
         buffer.append(i)

         if i.strip() == '.':
            start = elapsed = 0
            meter += 1

            from counters import counter

            TypeId, ConMatrix = init(buffer)
            buffer = []

            neighbors = {}

            start = time.process_time() # or perf_counter() 
            
            from findParaScen import findParaScen
            parascenario = findParaScen(ConMatrix, neighbors, args.pc)

            elapsed = time.process_time() - start
            total += elapsed
            times.append(elapsed)
            
            satcalls.append(counter['oracle'])

            from basesplit import bsplit as split
            plevel = sum([(split[parascenario[i][j]-1][0]-1) for i in range(len(ConMatrix)) for j in range(i+1, len(ConMatrix))])
            plevels.append(plevel)
           
            # print values of complexity counters
            print('--------------------------------------')
            print('Type ID: ', TypeId)
            print('--------------------------------------')
            print('Paraconsistency level: ', plevel)
            print('SAT calls: ', counter['oracle'])
            print('Visited nodes: ', counter['nodes'])
            print('Decisions: ', counter['decisions'])
            #print('Visited arcs: ', counter['arcs'])
            #print('Checked constraints: ', counter['cons'])
            print('--------------------------------------')
            print('Time: ', elapsed, 'sec')
            print('--------------------------------------')
            
            decs.append(counter['decisions'])
            nods.append(counter['nodes'])

            counter['arcs'] = 0
            counter['cons'] = 0
            counter['nodes'] = 0
            counter['decisions'] = 0
            counter['oracle'] = 0

            # print parascenario  
            if args.printsol == True:
               from printm import printMatrix
               printMatrix(parascenario)
               print('--------------------------------------')

            ConMatrix = None
          
   print('======================================')
   print('Total time:', total, 'sec')
   print('Min | Avg | Max time per CSP:', min(times), '|', sum(times)/(1.0*meter), '|', max(times))
   print('Min | Avg | Max paraconsistency levels:', min(plevels), '|', sum(plevels)/(1.0*meter), '|', max(plevels))
   print('Min | Avg | Max SAT calls:', min(satcalls), '|', sum(satcalls)/(1.0*meter), '|', max(satcalls))
   print('Min | Avg | Max decisions :', min(decs), '|', sum(decs)/(1.0*meter), '|', max(decs))
   print('Min | Avg | Max nodes visited:', min(nods), '|', sum(nods)/(1.0*meter), '|', max(nods))
   print('======================================')

if __name__ == '__main__':
   sys.exit(main())

